'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.proxyReducer = exports.proxyGetState = exports.proxyDispatch = exports.proxyStateForUI = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; // 代理getUIState的第一个参数，
// eg 使用$connect构建的组件，实例化时传入prefix 为{A: 'tab1'},
// 那么getUIState在获取A model的时候，返回的是A.state.tab1下的属性和值

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var proxyStateForUI = exports.proxyStateForUI = function proxyStateForUI(st, prefix) {
    var state = _extends({}, st);
    var propertyNames = Object.getOwnPropertyNames(st);
    propertyNames.map(function (name) {
        var namespace = Object.keys(prefix).find(function (namespace) {
            return namespace === name;
        });
        if (namespace) {
            state[namespace] = st[namespace][prefix[namespace]];
        }
    });

    return state;
};

// dispatch时为action 附加一个参数prefix， prefix所在model根据action.type判断
// 使reducer可以在正确的位置更新状态
var proxyDispatch = exports.proxyDispatch = function proxyDispatch(dispatch, key, prefix) {
    return function (action) {
        var actionMeta = {
            _callback: key,
            prefix: prefix[action.type.split('/')[0]]

        };
        action.meta = _extends({}, actionMeta, action.meta || {});
        dispatch(action);
    };
};

// 拿到state[prefix]下的数据
var proxyGetState = exports.proxyGetState = function proxyGetState(getState, prefix) {
    return function () {
        var state = _extends({}, getState());

        Object.keys(prefix).forEach(function (namespace) {
            state[namespace] = state[namespace][prefix[namespace]];
        });

        return state;
    };
};

// 在state[prefix] 下更新数据
var proxyReducer = exports.proxyReducer = function proxyReducer(reducers) {
    return Object.keys(reducers).reduce(function (ret, key) {
        ret[key] = function (state, action) {
            var prefix = action.meta.prefix;
            var result = reducers[key](state[prefix], action);

            return _extends({}, state, _defineProperty({}, action.meta.prefix, _extends({}, result)));
        };
        return ret;
    }, {});
};